using System;

namespace JmBucknall.CsvParser {

  public class StringCharTokenizer : ICharTokenizer {
    private string s;
    private int index;
    private bool haveUnreadChar;
    private char unreadChar;

    public StringCharTokenizer(string s) {
      this.s = s;
      index = 0;
      haveUnreadChar = false;
    }

    private void skipCrInCrLf() {
      if ((s[index] == '\r') && (index + 1 < s.Length) && (s[index + 1] == '\n'))
        index++;
    }

    private char mapCrToLf(char c) {
      if (c == '\r')
        return '\n';
      return c;
    }

    public char Peek() {
      if (haveUnreadChar)
        return unreadChar;
      if (index < s.Length)
        return mapCrToLf(s[index]);
      return CsvConstants.EOF;
    }

    public char Read() {
      if (haveUnreadChar) {
        haveUnreadChar = false;
        return unreadChar;
      }
      if (index < s.Length) {
        skipCrInCrLf();
        return mapCrToLf(s[index++]);
      }
      return CsvConstants.EOF;
    }

    public void Unread(char c) {
      if (haveUnreadChar) {
        throw new CharTokenizerException("Unread() cannot accept more than one pushed back character");
      }
      haveUnreadChar = true;
      unreadChar = c;
    }
  }
}
